#include "opencv2/highgui/highgui_c.h"
#include "../../Include/iANPR.h"
#include <stdio.h>
#include <Windows.h>
#include <omp.h>


void printHelp (char* fullName)
{
	printf  ("Use: %s <type_number> <path to image>\n\n", fullName);	
	puts ("type_number: 7 for Russian, 104 for Kazakhstan, 203 for Turkmenistan, 300 for Belarus vehicle registration plates");
	puts ("For more type_numbers please refer to iANPR SDK documentation\n");
	printf ("Example: %s 7 C:\\test.jpg - recognition of russian vehicle registration plates from file test.jpg\n", fullName);	
}


int main( int argc, char** argv)
{
	IplImage* Img = 0;
	int typeNumber = -9999;
	
	// filter input
	if (argc < 2)
	{
		printf ("Too few arguments. For help print %s /?", argv [0]);
		return -1;
	}
	else if (!strcmp (argv [1], "help") || !strcmp (argv [1], "-help") || !strcmp (argv [1], "--help") || !strcmp (argv [1], "/?"))
	{
		printHelp (argv [0]);
		return 0;
	}	
	else if (argc < 3)
	{
		printf ("Too few arguments. For help print %s /?", argv [0]);
		return -2;
	}	

	bool isFullType = false;
	for (size_t i = 0; i < anprFullTypesCount; i++)
	if (anprFullTypes[i] == typeNumber)
		isFullType = true;

	// argc >= 3
	typeNumber = atoi (argv [1]);
	if (isFullType) 
		Img = cvLoadImage( argv[2], CV_LOAD_IMAGE_COLOR  );
	else
		Img = cvLoadImage( argv[2], CV_LOAD_IMAGE_GRAYSCALE );

	if (!Img)
	{
		printf( "Can't load file!\n");
		return -4;
	}
	
	//  LicenseCapture     
	//    ,   .	
	char* key = new char[8001]; memset(key, 0, 8001);
	FILE* f = fopen("lic.key", "rb");
	if (f != NULL)
	{
		fread((void*)key, 8000, 1, f);
		fclose(f);
	}
	else
		puts("WARNING! File lic.key not found. This may crash program if you use license version of iANPR SDK dlls");

	LicenseValue(key);
	delete[] key; key = 0;
	
	{
		// 
		CvRect Rects[100];
		int all = 100;
		char** res = new char*[all];
		for(int j=0;j<all;j++) res[j] = new char[20];
		ANPR_OPTIONS a;
		a.Detect_Mode = ANPR_DETECTCOMPLEXMODE;
		a.min_plate_size = 500;
		a.max_plate_size = 50000;
		a.max_text_size = 20;		
		a.type_number = typeNumber;		
		a.flags = 0;
		int i;		
		DWORD tick1 = GetTickCount();
		i = anprPlate( Img,  a, &all, Rects, res ); 
		DWORD tick2 = GetTickCount();			
		if ( i == 0 )
			for( int j = 0; j < all; j++ ) 
			{				
				printf( "%s\n", res[j] );
			}
			printf( "Time:%5.3f;\n", (float) ( tick2 - tick1) / 1000 );

			for(int j=0;j<100;j++) delete [] res[j];
			delete [] res;		
	}

	// 
	int add_size = 40; //  40    , 80 -   

	int max = omp_get_num_procs();	
	IplImage** Images = new IplImage*[max];
	int i;

	DWORD tick1 = GetTickCount();

	for( i = 0; i < max; i++ )
	{
		int k = add_size, h = 0;
		if ( i != 0 && i != max - 1 ) {
			k += add_size;
			h = add_size;
		}
		if ( i == max - 1 ) h = add_size;

		if (isFullType)
			Images[i] = cvCreateImage( cvSize( Img->width, k + Img->height / max ), 8, 3 );
		else
			Images[i] = cvCreateImage( cvSize( Img->width, k + Img->height / max ), 8, 1 );

		cvSetImageROI( Img, cvRect( 0, i * int(Img->height / max) - h, Img->width, Img->height / max + k ) );
		cvCopy( Img, Images[i] );
		cvResetImageROI( Img );
	}

#pragma omp parallel
	{
#pragma omp for
		for( i = 0; i < max; i++ )
		{
			CvRect Rects[100];
			int all = 100;
			char** res = new char*[all];
			for(int j=0;j<all;j++) res[j] = new char[20];
			ANPR_OPTIONS a;
			a.Detect_Mode = ANPR_DETECTCOMPLEXMODE;
			a.min_plate_size = 500;
			a.max_plate_size = 50000;
			a.max_text_size = 20;		
			a.type_number = typeNumber;		
			a.flags = 0;

			int i2 = anprPlate( Images[i],  a, &all, Rects, res ); 
			if ( i2 == 0 )
				for( int j = 0; j < all; j++ ) {				
#pragma omp critical
					{
						printf( "%s\n", res[j] );
					}
				}

				for(int j=0;j<100;j++) delete [] res[j];
				delete [] res;		
		}
	}
	DWORD tick2 = GetTickCount();
	printf( "Time:%5.3f;\n", (float) ( tick2 - tick1) / 1000 );

	for( i = 0; i < max; i++ )
		cvReleaseImage ( &Images[i] );
	delete [] Images;

	cvReleaseImage ( &Img );

	return 0;
}